package com.ahai.vgk_game_services

import android.app.Activity
import android.content.Intent
import android.util.Log
import android.view.Gravity
import android.util.Pair
import com.google.android.gms.auth.api.Auth
import com.google.android.gms.auth.api.signin.GoogleSignIn
import com.google.android.gms.auth.api.signin.GoogleSignInAccount
import com.google.android.gms.auth.api.signin.GoogleSignInClient
import com.google.android.gms.auth.api.signin.GoogleSignInOptions
import com.google.android.gms.games.AchievementsClient
import com.google.android.gms.games.Games
import com.google.android.gms.games.LeaderboardsClient
import com.google.android.gms.games.SnapshotsClient
import com.google.android.gms.games.SnapshotsClient.DataOrConflict
import com.google.android.gms.games.snapshot.Snapshot
import com.google.android.gms.games.snapshot.SnapshotMetadataChange
import com.google.android.gms.tasks.Continuation
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry
import io.flutter.plugin.common.PluginRegistry.ActivityResultListener

private const val CHANNEL_NAME = "vgk_game_services"
private const val RC_SIGN_IN = 9000
class VgkGameServicesPlugin(private var activity: Activity? = null) : FlutterPlugin, MethodCallHandler, ActivityAware, ActivityResultListener {

  private var googleSignInClient: GoogleSignInClient? = null
  private var achievementClient: AchievementsClient? = null
  private var leaderboardsClient: LeaderboardsClient? = null
  private var activityPluginBinding: ActivityPluginBinding? = null
  private var channel: MethodChannel? = null
  private var pendingOperation: PendingOperation? = null

  companion object {
    @JvmStatic
    fun registerWith(registrar: PluginRegistry.Registrar) {
      val channel = MethodChannel(registrar.messenger(), CHANNEL_NAME)
      val plugin = VgkGameServicesPlugin(registrar.activity())
      channel.setMethodCallHandler(plugin)
      registrar.addActivityResultListener(plugin)
    }
  }

  private fun silentSignIn(result: Result) {
    val activity = activity ?: return
    val builder = GoogleSignInOptions.Builder(
            GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
    googleSignInClient = GoogleSignIn.getClient(activity, builder.build())
    googleSignInClient?.silentSignIn()?.addOnCompleteListener { task ->
      pendingOperation = PendingOperation(Methods.silentSignIn, result)
      if (task.isSuccessful) {
        val googleSignInAccount = task.result ?: return@addOnCompleteListener
        handleSignInResult(googleSignInAccount)
      } else {
        Log.e("Error", "signInError", task.exception)
        Log.i("ExplicitSignIn", "Trying explicit sign in")
        explicitSignIn()
      }
    }
  }

  private fun explicitSignIn() {
    val activity = activity ?: return
    val builder = GoogleSignInOptions.Builder(
            GoogleSignInOptions.DEFAULT_GAMES_SIGN_IN)
            .requestEmail()
    googleSignInClient = GoogleSignIn.getClient(activity, builder.build())
    activity.startActivityForResult(googleSignInClient?.signInIntent, RC_SIGN_IN)
  }

  private fun handleSignInResult(googleSignInAccount: GoogleSignInAccount) {
    val activity = this.activity ?: return
    achievementClient = Games.getAchievementsClient(activity, googleSignInAccount)
    leaderboardsClient = Games.getLeaderboardsClient(activity, googleSignInAccount)

    val lastSignedInAccount = GoogleSignIn.getLastSignedInAccount(activity) ?: return
    val gamesClient = Games.getGamesClient(activity, lastSignedInAccount)
    gamesClient.setViewForPopups(activity.findViewById(android.R.id.content))
    gamesClient.setGravityForPopups(Gravity.TOP or Gravity.CENTER_HORIZONTAL)

    finishPendingOperationWithSuccess()
  }

  private val isSignedIn: Boolean get() {
    val activity = activity ?: return false
    return GoogleSignIn.getLastSignedInAccount(activity) != null
  }

  private fun getPlayer(result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    val activity = activity ?: return
    val lastSignedInAccount = GoogleSignIn.getLastSignedInAccount(activity) ?: return
    Games.getPlayersClient(activity, lastSignedInAccount)
      .getCurrentPlayer().addOnSuccessListener { player -> 
        val hashMap:HashMap<String,String> = HashMap<String,String>()
        hashMap.put("Name",player.getDisplayName())  
        hashMap.put("PlayerId",player.getPlayerId())  
        val playerLevelInfo = player.getLevelInfo()
        val currentLevel ="0"
        playerLevelInfo?.let {
          val level= playerLevelInfo.getCurrentLevel()
          level?.let{
            val levelNumber= level.getLevelNumber()
            levelNumber?.let {
              hashMap.put("currentLevel", levelNumber.toString()) 
            }
          }
        }
        result.success(hashMap)
      }.addOnFailureListener {
        result.error("error", it.localizedMessage, null)
      }
  }

  private fun saveSnapshot(
    gameName: String,
    dataToSave: String,
    description: String,
    result: Result
  ) {
      showLoginErrorIfNotLoggedIn(result)
      val lastSignedInAccount = GoogleSignIn.getLastSignedInAccount(activity) ?: return
      val snapshotsClient = Games.getSnapshotsClient(activity, lastSignedInAccount)
      val conflictResolutionPolicy = SnapshotsClient.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED
      snapshotsClient.open(gameName, true, conflictResolutionPolicy)
          .addOnFailureListener {
            result.error("error", it.localizedMessage, null)
          }
          .continueWith<Pair<Snapshot, ByteArray>>(
              Continuation<DataOrConflict<Snapshot>, Pair<Snapshot, ByteArray>> { task ->
                  val snapshot = task.result
                  snapshot?.data?.let {
                      return@Continuation Pair(it, toByteArray(dataToSave))
                  }
                  null
              })
          .addOnCompleteListener { task ->
              if (task.isSuccessful && task.result != null) {
                  val snapshot = task.result!!.first
                  val data = task.result!!.second
                  snapshot.snapshotContents.writeBytes(data)
                  val metadataChange = SnapshotMetadataChange.Builder()
                  .setDescription(description)
                  .build()
                  val commitTask = snapshotsClient.commitAndClose(snapshot, metadataChange)
                  commitTask.addOnCompleteListener { task1 ->
                      if (task1.isSuccessful) {
                          result.success("success")
                      } else {
                          result.error("error", null, null)
                      }
                  }
              } else {
                result.error("error", it.localizedMessage, null)
              }
          }
  }

  private fun loadSnapshot(gameName: String, result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    val lastSignedInAccount = GoogleSignIn.getLastSignedInAccount(activity) ?: return
    val snapshotsClient = Games.getSnapshotsClient(activity, lastSignedInAccount)
    val conflictResolutionPolicy = SnapshotsClient.RESOLUTION_POLICY_MOST_RECENTLY_MODIFIED
    snapshotsClient.open(gameName, true, conflictResolutionPolicy)
    .addOnFailureListener {
      result.error("error", it.localizedMessage, null)
    }
    .continueWith<ByteArray>(Continuation { task ->
        val snapshot = task.result
        try {
            snapshot?.data?.let {
                result.success(@Continuation it.snapshotContents.readFully())
            }
            result.error("error", it.localizedMessage, null)
        } catch (e: IOException) {
          result.error("error", it.localizedMessage, null)
        } 
    })
    .addOnCompleteListener { task ->
        if (task.isSuccessful) {
            task.result?.let {
                val data = toStringData(it)
                result.success(data)
            }
        } else {
          result.error("error", it.localizedMessage, null)
        }
    }
  }

  private fun signOut(result: Result) {
    googleSignInClient?.signOut()?.addOnCompleteListener { task ->
      if (task.isSuccessful) {
        result.success("success")
      } else {
        result.error("error", "${task.exception}", null)
      }
    }
  }

  private fun showAchievements(result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    achievementClient?.achievementsIntent?.addOnSuccessListener { intent ->
      activity?.startActivityForResult(intent, 0)
      result.success("success")
    }?.addOnFailureListener {
      result.error("error", "${it.message}", null)
    }
  }
  
  private fun toByteArray(data: String): ByteArray {
    return data.toByteArray()
  }

  private fun toStringData(bytes: ByteArray): String {
    return String(bytes)
  }

  private fun unlock(achievementID: String, result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    achievementClient?.unlockImmediate(achievementID)?.addOnSuccessListener {
      result.success("success")
    }?.addOnFailureListener {
      result.error("error", it.localizedMessage, null)
    }
  }

  private fun increment(achievementID: String, count: Int, result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    achievementClient?.incrementImmediate(achievementID, count)
            ?.addOnSuccessListener {
              result.success("success")
            }?.addOnFailureListener {
              result.error("error", it.localizedMessage, null)
            }
  }

  private fun showLeaderboards(leaderboardID: String, result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    val onSuccessListener: ((Intent) -> Unit) = { intent ->
      activity?.startActivityForResult(intent, 0)
      result.success("success")
    }
    val onFailureListener: ((Exception) -> Unit) = {
      result.error("error", "${it.message}", null)
    }
    if (leaderboardID.isEmpty()) {
      leaderboardsClient?.allLeaderboardsIntent
        ?.addOnSuccessListener(onSuccessListener)
        ?.addOnFailureListener(onFailureListener)
    } else {
      leaderboardsClient
        ?.getLeaderboardIntent(leaderboardID)
        ?.addOnSuccessListener(onSuccessListener)
        ?.addOnFailureListener(onFailureListener)
    }
  }

  private fun submitScore(leaderboardID: String, score: Int, result: Result) {
    showLoginErrorIfNotLoggedIn(result)
    leaderboardsClient?.submitScoreImmediate(leaderboardID, score.toLong())?.addOnSuccessListener {
      result.success("success")
    }?.addOnFailureListener {
      result.error("error", it.localizedMessage, null)
    }
  }

  private fun showLoginErrorIfNotLoggedIn(result: Result) {
    if (achievementClient == null || leaderboardsClient == null) {
      result.error("error", "Please make sure to call signIn() first", null)
    }
  }

  override fun onAttachedToEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    setupChannel(binding.binaryMessenger)
  }

  override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
    teardownChannel()
  }

  private fun setupChannel(messenger: BinaryMessenger) {
    channel = MethodChannel(messenger, CHANNEL_NAME)
    channel?.setMethodCallHandler(this)
  }

  private fun teardownChannel() {
    channel?.setMethodCallHandler(null)
    channel = null
  }

  private fun disposeActivity() {
    activityPluginBinding?.removeActivityResultListener(this)
    activityPluginBinding = null
  }

  override fun onDetachedFromActivity() {
    disposeActivity()
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    onAttachedToActivity(binding)
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activityPluginBinding = binding
    activity = binding.activity
    binding.addActivityResultListener(this)
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  private class PendingOperation constructor(val method: String, val result: Result)

  private fun finishPendingOperationWithSuccess() {
    Log.i(pendingOperation?.method, "success")
    pendingOperation?.result?.success("success")
    pendingOperation = null
  }

  private fun finishPendingOperationWithError(errorMessage: String) {
    Log.i(pendingOperation?.method, "error")
    pendingOperation?.result?.error("error", errorMessage, null)
    pendingOperation = null
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?): Boolean {
    if (requestCode == RC_SIGN_IN) {
      val result = data?.let { Auth.GoogleSignInApi.getSignInResultFromIntent(it) }
      val signInAccount = result?.signInAccount
      if (result?.isSuccess == true && signInAccount != null) {
        handleSignInResult(signInAccount)
      } else {
        var message = result?.status?.statusMessage ?: ""
        if (message.isEmpty()) {
          message = "Something went wrong " + result?.status
        }
        finishPendingOperationWithError(message)
      }
      return true
    }
    return false
  }

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      Methods.unlock -> {
        unlock(call.argument<String>("achievementID") ?: "", result)
      }
      Methods.increment -> {
        val achievementID = call.argument<String>("achievementID") ?: ""
        val steps = call.argument<Int>("steps") ?: 1
        increment(achievementID, steps, result)
      }
      Methods.submitScore -> {
        val leaderboardID = call.argument<String>("leaderboardID") ?: ""
        val score = call.argument<Int>("value") ?: 0
        submitScore(leaderboardID, score, result)
      }
      Methods.showLeaderboards -> {
        val leaderboardID = call.argument<String>("leaderboardID") ?: ""
        showLeaderboards(leaderboardID, result)
      }
      Methods.showAchievements -> {
        showAchievements(result)
      }
      Methods.silentSignIn -> {
        silentSignIn(result)
      }
      Methods.isSignedIn -> {
        result.success(isSignedIn)
      }
      Methods.signOut -> {
        signOut(result)
      }
      Methods.getPlayer -> {
        getPlayer(result)
      }
      Methods.saveSnapshot -> {
        val gameName = call.argument<String>("gameName") ?: ""
        val dataToSave = call.argument<String>("dataToSave") ?: ""
        val description = call.argument<String>("description") ?: ""
        saveSnapshot(gameName, dataToSave, description, result)
      }
      Methods.loadSnapshot -> {
        val gameName = call.argument<String>("gameName") ?: ""
        loadSnapshot(gameName, result)
      }
      else -> result.notImplemented()
    }
  }
}

object Methods {
  const val unlock = "unlock"
  const val increment = "increment"
  const val submitScore = "submitScore"
  const val showLeaderboards = "showLeaderboards"
  const val showAchievements = "showAchievements"
  const val silentSignIn = "silentSignIn"
  const val isSignedIn = "isSignedIn"
  const val getPlayer = "getPlayer"
  const val saveSnapshot = "saveSnapshot"
  const val loadSnapshot = "loadSnapshot"
  const val signOut = "signOut"
}
