import 'dart:async';

import 'package:plugin_platform_interface/plugin_platform_interface.dart';
import 'package:vgk_game_services/games_services_platform_interface/method_channel_games_services.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/access_point_location.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/achievement.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/result.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/score.dart';

abstract class GamesServicesPlatform extends PlatformInterface {
  GamesServicesPlatform() : super(token: _token);

  static final Object _token = Object();

  static GamesServicesPlatform _instance = MethodChannelGamesServices();

  static GamesServicesPlatform get instance => _instance;

  static set instance(GamesServicesPlatform instance) {
    PlatformInterface.verifyToken(instance, _token);
    _instance = instance;
  }

  Future<Result> increment({achievement = Achievement}) async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> unlock({achievement = Achievement}) async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> submitScore({score = Score}) async {
    throw UnimplementedError("not implemented.");
  }

  /// It will open the achievements screen.
  Future<Result> showAchievements() async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> showLeaderboards(
      {iOSLeaderboardID = "", androidLeaderboardID = ""}) async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> signIn() async {
    throw UnimplementedError("not implemented.");
  }

  Future<bool?> get isSignedIn => throw UnimplementedError("not implemented.");

  Future<Result> signOut() async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> showAccessPoint(AccessPointLocation location) async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> hideAccessPoint() async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> getPlayer() async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> loadSnapshot({gameName = ""}) async {
    throw UnimplementedError("not implemented.");
  }

  Future<Result> saveSnapshot(
      {gameName = "", dataToSave = "", description = ""}) async {
    throw UnimplementedError("not implemented.");
  }
}
