import 'dart:async';

import 'package:vgk_game_services/games_services_platform_interface/game_services_platform_interface.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/access_point_location.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/achievement.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/result.dart';
import 'package:vgk_game_services/games_services_platform_interface/models/score.dart';

class VgkGameServices {
  static Future<Result> unlock({achievement = Achievement}) async {
    return await GamesServicesPlatform.instance
        .unlock(achievement: achievement);
  }

  static Future<Result> increment({achievement = Achievement}) async {
    return await GamesServicesPlatform.instance
        .increment(achievement: achievement);
  }

  static Future<Result> submitScore({score = Score}) async {
    return await GamesServicesPlatform.instance.submitScore(score: score);
  }

  static Future<Result> showAchievements() async {
    return await GamesServicesPlatform.instance.showAchievements();
  }

  static Future<Result> showLeaderboards(
      {iOSLeaderboardID = "", androidLeaderboardID = ""}) async {
    return await GamesServicesPlatform.instance.showLeaderboards(
        iOSLeaderboardID: iOSLeaderboardID,
        androidLeaderboardID: androidLeaderboardID);
  }

  static Future<Result> signIn() async {
    return await GamesServicesPlatform.instance.signIn();
  }

  static Future<bool> get isSignedIn async =>
      await GamesServicesPlatform.instance.isSignedIn ?? false;

  static Future<Result> signOut() async {
    return await GamesServicesPlatform.instance.signOut();
  }

  static Future<Result> showAccessPoint(AccessPointLocation location) async {
    return await GamesServicesPlatform.instance.showAccessPoint(location);
  }

  static Future<Result> hideAccessPoint() async {
    return await GamesServicesPlatform.instance.hideAccessPoint();
  }

  static Future<Result> getPlayer() async {
    return await GamesServicesPlatform.instance.getPlayer();
  }

  static Future<Result> loadSnapshot({gameName = ""}) async {
    return await GamesServicesPlatform.instance
        .loadSnapshot(gameName: gameName);
  }

  static Future<Result> saveSnapshot(
      {gameName = "", dataToSave = "", description = ""}) async {
    return await GamesServicesPlatform.instance.saveSnapshot(
      gameName: gameName,
      dataToSave: dataToSave,
      description: description,
    );
  }
}
